import { ImageCropStrategy } from "upload-image-plugin/params/ImageCropStrategy";

/**
 * Geometry defining an image resize.
 *
 * Note: Some resize modes automatically crop to maintain an aspect ratio without distorting the image.
 */
export interface ResizeGeometry {
  size: ImageSize;
}

/**
 * Geometry defining an image crop.
 */
export interface CropGeometry {
  offset: {
    /**
     * @description
     * Query Param: crop-x
     */
    x: number;

    /**
     * @description
     * Query Param: crop-y
     */
    y: number;
  };
  size: {
    /**
     * @description
     * Query Param: crop-h
     */
    height: number;

    /**
     * Legacy. Retained for validation purposes and to document existing data structures.
     */
    type: "widthxheight!" | undefined;

    /**
     * @description
     * Query Param: crop-w
     */
    width: number;
  };
}

/**
 * Image size.
 */
export type ImageSize =
  | ImageSizeWidth
  | ImageSizeHeight
  | ImageSizeWidthHeightMax
  | ImageSizeWidthHeightMin
  | ImageSizeWidthHeightStretch
  | ImageSizeWidthHeightShrink
  | ImageSizeWidthHeightCrop;

/**
 * Width given, height automagically selected to preserve aspect ratio.
 */
export interface ImageSizeWidth {
  /**
   * @description
   * Query Param: fit
   * Query Param Value: width
   * Query Param Precedence: 0
   */
  type: "width";

  /**
   * @description
   * Query Param: w
   */
  width: number;
}

/**
 * Height given, width automagically selected to preserve aspect ratio.
 */
export interface ImageSizeHeight {
  /**
   * @description
   * Query Param: h
   */
  height: number;

  /**
   * @description
   * Query Param: fit
   * Query Param Value: height
   * Query Param Precedence: 1
   */
  type: "xheight";
}

/**
 * Maximum values of height and width given, aspect ratio preserved.
 */
export interface ImageSizeWidthHeightMax {
  /**
   * @description
   * Query Param: h
   */
  height: number;

  /**
   * @description
   * Query Param: fit
   * Query Param Value: max
   * Query Param Precedence: 2
   */
  type: "widthxheight";

  /**
   * @description
   * Query Param: w
   */
  width: number;
}

/**
 * Minimum values of width and height given, aspect ratio preserved.
 */
export interface ImageSizeWidthHeightMin {
  /**
   * @description
   * Query Param: h
   */
  height: number;

  /**
   * @description
   * Query Param: fit
   * Query Param Value: min
   * Query Param Precedence: 3
   */
  type: "widthxheight^";

  /**
   * @description
   * Query Param: w
   */
  width: number;
}

/**
 * Aspect ratio preserved, image cropped if required. Position dictates cropping strategy.
 */
export interface ImageSizeWidthHeightCrop {
  cropStrategy: ImageCropStrategy;

  /**
   * @description
   * Query Param: h
   */
  height: number;

  /**
   * @description
   * Query Param: fit
   * Query Param Value: crop
   * Query Param Precedence: 4
   */
  type: "widthxheightc";

  /**
   * @description
   * Query Param: w
   */
  width: number;
}

/**
 * Width and height emphatically given, original aspect ratio ignored.
 */
export interface ImageSizeWidthHeightStretch {
  /**
   * @description
   * Query Param: h
   */
  height: number;

  /**
   * @description
   * Query Param: fit
   * Query Param Value: stretch
   * Query Param Precedence: 5
   */
  type: "widthxheight!";

  /**
   * @description
   * Query Param: w
   */
  width: number;
}

/**
 * Shrinks an image with dimension(s) larger than the corresponding width and/or height argument(s).
 */
export interface ImageSizeWidthHeightShrink {
  /**
   * @description
   * Query Param: h
   */
  height: number;

  /**
   * @description
   * Query Param: fit
   * Query Param Value: shrink
   * Query Param Precedence: 6
   */
  type: "widthxheight>";

  /**
   * @description
   * Query Param: w
   */
  width: number;
}
